#!/usr/bin/python3
import os
import re
import subprocess
import sys
import time

import dbus
from pydbus import SystemBus
from gi.repository import GLib

loop = GLib.MainLoop()


class MyException(Exception):
    pass


class TestObject(object):
    dbus = '''
    <node>
        <interface name='org.redos.edu.pydbus.RedosEdu'>
            <method name='granteAuth'>
                <arg type='b' name='interactive' direction='in'/>
                <arg type='s' name='response' direction='out'/>
            </method>
            <method name='dependencies_check'>
                <arg type='s' name='package_name' direction='in'/>
                <arg type='s' name='version' direction='in'/>
                <arg type='s' name='epoch' direction='in'/>
                <arg type='s' name='release_ver' direction='in'/>
                <arg type='s' name='command' direction='in'/>
                <arg type='s' name='response' direction='out'/>
            </method>
            <method name='install'>
                <arg type='s' name='package_name' direction='in'/>
                <arg type='s' name='version' direction='in'/>
                <arg type='s' name='epoch' direction='in'/>
                <arg type='s' name='release_ver' direction='in'/>
                <arg type='s' name='response' direction='out'/>
            </method>
            <method name='local_install'>
                <arg type='s' name='rpm_path' direction='in'/>
                <arg type='s' name='response' direction='out'/>
            </method>
            <method name='remove'>
                <arg type='s' name='package_name' direction='in'/>
                <arg type='s' name='version' direction='in'/>
                <arg type='s' name='epoch' direction='in'/>
                <arg type='s' name='release_ver' direction='in'/>
                <arg type='s' name='response' direction='out'/>
            </method>
            <method name='update'>
                <arg type='s' name='package_name' direction='in'/>
                <arg type='s' name='version' direction='in'/>
                <arg type='s' name='epoch' direction='in'/>
                <arg type='s' name='release_ver' direction='in'/>
                <arg type='s' name='response' direction='out'/>
            </method>
            <method name='repomanager'>
                <arg type='s' name='repoid' direction='in'/>
                <arg type='s' name='enable' direction='in'/>
            </method>
            <method name='quit'/>
            <method name='status_auth'>
                <arg type='s' name='enable' direction='out'/>
            </method>
            <method name='dependencies_all_update'>
                <arg type='s' name='response' direction='out'/>
            </method>
            <method name='dependencies_local_check'>
                <arg type='s' name='rpm_path' direction='in'/>
                <arg type='s' name='response' direction='out'/>
            </method>
            <method name='update_all'>
                <arg type='s' name='response' direction='out'/>
            </method>
            <method name='dependencies_group_action'>
                <arg type='s' name='install_specs' direction='in'/>
                <arg type='s' name='remove_specs' direction='in'/>
                <arg type='as' name='response' direction='out'/>
            </method>
            <method name='group_install'>
                <arg type='s' name='spec' direction='in'/>
                <arg type='s' name='response' direction='out'/>
            </method>
            <method name='group_remove'>
                <arg type='s' name='spec' direction='in'/>
                <arg type='s' name='response' direction='out'/>
            </method>
        </interface>
    </node>
    '''

    def granteAuth(self, interactive,
                   dbus_context):
        # if dbus_context.is_authorized('org.freedesktop.policykit.exec', {'polkit.icon': 'abcd', 'aaaa': 'zzzz'},
        #                               interactive=interactive):
            global rules_granted
            self.rules_granted = True
            return "rules granted"
        # else:
        #     self.rules_granted = False
        #     return "rules not granted"

    def repomanager(self, repoid, enable):
        if self.rules_granted:
            if enable == "1":
                key = "--set-enabled"
            else:
                key = "--set-disabled"
            print(['/usr/bin/dnf', 'config-manager', key, repoid])
            dnf_out = subprocess.Popen(['/usr/bin/dnf', 'config-manager', key, repoid],
                                       stdout=subprocess.PIPE,
                                       stderr=subprocess.PIPE)
            output, error = dnf_out.communicate()
            if dnf_out.returncode != 0:
                return str(f"{error.decode('utf-8')}")
            else:
                return str("0")
        else:
            return "rules not granted"

    def status_auth(self):
        if hasattr(self, 'rules_granted'):
            if self.rules_granted:
                return "True"
            else:
                return "False"
        else:
            return "False"

    def install(self, package_name, version, epoch, release_ver):
        if self.rules_granted:
            spec = f"{package_name}-{epoch}:{version}-{release_ver}" if str(epoch) != "None" else \
                f"{package_name}-{version}-{release_ver}"
            return self.dnf_command("install", spec)
        else:
           return "rules not granted"

    def local_install(self, rpm_path):
        if self.rules_granted:
            spec = rpm_path
            return self.dnf_command("install", spec)
        else:
            return "rules not granted"

    def dependencies_check(self, package_name, version, epoch, release_ver, command):
        if self.rules_granted:
            spec = f"{package_name}-{epoch}:{version}-{release_ver}" if str(epoch) != "None" else \
                f"{package_name}-{version}-{release_ver}"
            if command == "update":
                spec = f"{package_name}"
            return self.dnf_resolvedep(command, spec)
        else:
            return "rules not granted"

    def dependencies_group_action(self, install_specs, remove_specs):
        if self.rules_granted:
            if install_specs != "" and remove_specs != "":
                return [self.dnf_resolvedep("install", install_specs), self.dnf_resolvedep("remove", remove_specs)]
            elif install_specs != "" and remove_specs == "":
                return [self.dnf_resolvedep("install", install_specs), "None"]
            elif remove_specs != "" and install_specs == "":
                return ["None", self.dnf_resolvedep("remove", remove_specs)]
        else:
            return ["rules not granted"]


    def dependencies_local_check(self, rpm_path):
        return self.dnf_resolvedep("install", rpm_path)

    def dependencies_all_update(self):
        dnf_out = subprocess.Popen(['/usr/bin/dnf', 'update', '--assumeno'],
                                   stdout=subprocess.PIPE,
                                   stderr=subprocess.PIPE)
        output, error = dnf_out.communicate()
        output = output.decode('utf-8').split('\n')[5:]
        output = output[:len(output) - 7]
        out = ""
        for o in output:
            out += f"{o}\n"
        return f"{out}|||{error.decode('utf-8')}"

    def group_remove(self, spec):
        if self.rules_granted:
            return self.dnf_command("remove", spec)
        else:
            return "rules not granted"

    def group_install(self, spec):
        if self.rules_granted:
            return self.dnf_command("install", spec)
        else:
            return "rules not granted"

    def update_all(self):
        if self.rules_granted:
            return self.dnf_command("update", "")
        else:
            return "rules not granted"

    def remove(self, package_name, version, epoch, release_ver):
        if self.rules_granted:
            spec = f"{package_name}-{epoch}:{version}-{release_ver}" if str(epoch) != "None" else \
                f"{package_name}-{version}-{release_ver}"
            return self.dnf_command("remove", spec)
        else:
            return "rules not granted"

    def update(self, package_name, version, epoch, release_ver):
        if self.rules_granted:
            spec = f"{package_name}"
            return self.dnf_command("update", spec)
        else:
            return "rules not granted"

    def dnf_resolvedep(self, command, spec):
        dnf_out = subprocess.Popen([f'/usr/bin/dnf {command} --assumeno {spec}'],
                               stdout=subprocess.PIPE,
                               stderr=subprocess.PIPE, shell=True)
        output, error = dnf_out.communicate()
        output = output.decode('utf-8').split('\n')
        if len(output) > 11:
            output = output[4:] if command == 'remove' else \
                output[5:]
            output = output[:len(output) - 7]
        out = ""
        for o in output:
            out += f"{o}\n"
        return f"{out}|||{error.decode('utf-8')}"

    def dnf_command(self, command, spec):
        if spec != "":
            dnf_out = subprocess.Popen([f'/usr/bin/dnf {command} -y {spec}'],
                                       stdout=subprocess.PIPE,
                                       stderr=subprocess.PIPE, shell=True)
        else:
            dnf_out = subprocess.Popen(['/usr/bin/dnf', command, '-y'],
                                       stdout=subprocess.PIPE,
                                       stderr=subprocess.PIPE)
        output, error = dnf_out.communicate()
        if dnf_out.returncode != 0:
            return f"{error.decode('utf-8')}"
        else:
            return "0"

    def quit(self):
        """removes this object from the DBUS connection and exits"""
        loop.quit()


if __name__ == "__main__":
    if len(sys.argv) == 1:
        with SystemBus() as bus:
            with bus.publish("org.redos.edu.pydbus.RedosEdu", TestObject()):
                loop.run()
    elif len(sys.argv) > 1:
        if sys.argv[1] == "--start":
            with SystemBus() as bus:
                with bus.publish("org.redos.edu.pydbus.RedosEdu", TestObject()):
                    loop.run()
        elif sys.argv[1] == "--stop":
            os.system("rm -rf /tmp/firebird")
            loop.quit()
